"""
Base Class for DesktopEntry, IconTheme and IconData

Currently not supported:
- Does not check if regex is valid
- Does not check if URL is valid

TODO:
- Check for double Keys / Groups
"""

import string, codecs, re
from Exceptions import *

class IniFile:
	content = {}
	defaultGroup = ''
	fileExtension = ''

	locale_LANG = 'C'
	locale_COUNTRY = ''
	locale_ENCODING = ''
	locale_MODIFIER = ''

	file = ''

	def __init__(self):
		# reset content
		self.content = dict()

	def parse(self, file, headers):
		# check file extension
		self.fileExtension = re.sub(".*\.", "", file)

		# parse file
		try:
			f = codecs.open(file, 'r', 'utf-8')
			self.file = file
		except IOError:
			raise ParsingError("File not found", file)

		try:
			lines = f.readlines()
		except UnicodeError:
			raise ParsingError("File contains non UTF-8 chars", file)

		for line in lines:
			# empty line
			if line.strip() == '':
				pass
			# comment
			elif line[0] == '#':
				pass
			# new group
			elif line[0] == '[':
				currentGroup = string.strip(line).strip("[").strip("]")
				self.content[currentGroup] = {}
			# key
			else:
				try:
					tmp = string.split(line, '=')
					tmp[0] = string.strip(tmp[0])
					tmp[1] = string.strip(tmp[1])
				except IndexError:
					raise ParsingError("Invalid Key=Value pair: " + line, file)
				try:
					self.content[currentGroup][tmp[0]] = tmp[1]
				except IndexError:
					raise ParsingError("["+headers[0]+"]-Header missing", file)

		# check header
		for header in headers:
			if self.content.has_key(header):
				self.defaultGroup = header
				break
		else:
			raise ParsingError("["+headers[0]+"]-Header missing", file)

	def getFileName(self):
		return self.file

	# start stuff to access the keys
	def get(self, key, type = "string", locale = False, list = False, group = ""):
		value = self.getKey(key, group, locale = locale)

	def get(self, key, group = "", locale = False, type = "string", list = False):
		# set default group
		if not group:
			group = self.defaultGroup

		# does Group exists?
		if not self.content.has_key(group):
			if debug:
				raise NoGroupError(group, self.file)
			else:
				return ""

		# does key exists?
		if not self.content[group].has_key(key):
			if debug:
				raise NoKeyError(key, group, self.file)
			else:
				return ""

		# return key (with locale)
		if locale:
			value = self.content[group][self.__addLocale(key, group)]
		else:
			value = self.content[group][key]

		if list == True:
			value = self.__getList(value)
		if type == "boolean":
			value = self.__getBoolean(value)
		elif type == "integer":
			value = int(value)
		elif type == "numeric":
			value = float(value)

		return value

	def hasGroup(self, group):
		if self.content.has_key(group):
			return True
		else:
			return False

	def hasKey(self, key, group = ''):
		if not group:
			group = self.defaultGroup

		if self.hasGroup(group):
			if self.content[group].has_key(key):
				return True
			else:
				return False
		else:
			return False
	# end stuff to access the keys

	# start subget
	def __getList(self, string):
		if re.search(',', string):
			return string.split(',')
		else:
			return string.split(';')

	def __getBoolean(self, boolean):
		if boolean == 1:
			return True
		elif boolean == 0:
			return False
		else:
			return string.capitalize(boolean)
	# end subget

	# start locale [] stuff
	def setLocale(self, lc_messages):
		"set locale for current desktop entry"
		# valid lc_messages format?
		p = re.compile('^([a-zA-Z]+)(_[a-zA-Z]+)?(\.[a-zA-Z\-0-9]+)?(@[a-zA-Z]+)?$')
		m = p.match(lc_messages)
		if not m:
			if debug:
				raise ValueError('Invalid LC_MESSAGES value')
			else:
				return
		# set lc_messages
		else:
			self.locale_LANG     = m.group(1)
			self.locale_COUNTRY  = string.replace(m.group(2) or '', '_', '')
			self.locale_ENCODING = string.replace(m.group(3) or '', '.', '')
			self.locale_MODIFIER = string.replace(m.group(4) or '', '@', '')

	def getLocale(self):
		"return locale in lc_messages format"
		lc_messages = self.locale_LANG
		if self.locale_COUNTRY:
			lc_messages = lc_messages + '_' + self.locale_COUNTRY
		if self.locale_ENCODING:
			lc_messages = lc_messages + '.' + self.locale_ENCODING
		if self.locale_MODIFIER:
			lc_messages = lc_messages + '@' + self.locale_MODIFIER
		return lc_messages

	def __addLocale(self, key, group = ''):
		"add locale to key according the current lc_messages"
		# set default group
		if not group:
			group = self.defaultGroup

		# add locale
		if (self.locale_LANG and self.locale_COUNTRY and self.locale_MODIFIER) and \
		(self.content[group].has_key(key+'['+self.locale_LANG+'_'+self.locale_COUNTRY+'@'+self.locale_MODIFIER+']')):
			return key+'['+self.locale_LANG+'_'+self.locale_COUNTRY+'@'+self.locale_MODIFIER+']'
		elif (self.locale_LANG and self.locale_COUNTRY) and \
		(self.content[group].has_key(key+'['+self.locale_LANG+'_'+self.locale_COUNTRY+']')):
			return key+'['+self.locale_LANG+'_'+self.locale_COUNTRY+']'
		elif (self.locale_LANG and self.locale_MODIFIER) and \
		(self.content[group].has_key(key+'['+self.locale_LANG+'@'+self.locale_MODIFIER+']')):
			return key+'['+self.locale_LANG+'@'+self.locale_MODIFIER+']'
		elif (self.locale_LANG) and \
		(self.content[group].has_key(key+'['+self.locale_LANG+']')):
			return key+'['+self.locale_LANG+']'
		else:
			return key
	# end locale stuff

	# start validation stuff
	def validate(self, report = "All"):
		"validate ... report = All / Warnings / Errors"
		self.warnings = []
		self.errors = []

		# overwrite this for own checkings
		self.checkExtras()

		# check all keys
		for group in self.content:
			self.checkGroup(group)
			for key in self.content[group]:
				self.checkKey(key, self.content[group][key], group)
				# check if value is empty
				if self.content[group][key] == "":
					self.warnings.append("Value of Key '%s' is empty" % key)

		# raise Warnings / Errors
		msg = ""

		if report == "All" or report == "Warnings":
			for line in self.warnings:
				msg += "\n- " + line

		if report == "All" or report == "Errors":
			for line in self.errors:
				msg += "\n- " + line

		if msg:
			raise ValidationError(msg, self.file)

	# check if group header is valid
	def checkGroup(self, group):
		pass

	# check if key is valid
	def checkKey(self, key, value, group):
		pass

	# check random stuff
	def checkExtras(self):
		pass

	def checkBoolean(self, key, value):
		# 1 or 0 : deprecated
		if (value == "1" or value == "0"):
			self.warnings.append("Value of key '%s' is a deprecated boolean value" % key)
		# true or false: ok
		elif not (value == "true" or value == "false"):
			self.errors.append("Value of key '%s' is not a boolean" % key)

	def checkNumber(self, key, value):
		# float() ValueError
		try:
			float(value)
		except:
			self.errors.append("Value of key '%s' is not a number" % key)

	def checkInteger(self, key, value):
		# int() ValueError
		try:
			int(value)
		except:
			self.errors.append("Value of key '%s' is not an integer" % key)

	def checkPoints(self, key, value):
		if not re.match("^[0-9]+,[0-9]+|[0-9]+,[0-9]+|[0-9]+,[0-9]+|[0-9]+,[0-9]+$", value):
			self.errors.append("Value of key '%s' is not a point value" % key)

	def checkString(self, key, value):
		# convert to ascii
		if not value.encode("ascii", 'ignore') == value:
			self.errors.append("Value of key '%s' must only contain ASCII characters" % key)
